/****************************************************************************
 **
 ** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights.  These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/

#include <QApplication>

#include <QObject>
#include <QShortcut>

#include "mainWindow.h"
#include "drawArea.h"
#include "version.h"

MainWindow::MainWindow()
{
    drawArea = new DrawArea;
    setCentralWidget(drawArea);

    exit = new QShortcut(Qt::Key_Escape, this);
    connect(exit, SIGNAL(activated()), this, SLOT(close()));

    connect(drawArea, SIGNAL(touchHold()), this, SLOT(close()));

    clear = new QShortcut(tr("Ctrl+N"), this);
    connect(clear, SIGNAL(activated()), drawArea, SLOT(clear()));

    QString windowTitle = tr("TouchTest ");
    QString versionString = VERSION_STRING;

    // use QRegExp to simultaniously validate the format and capture the version
    // number from the version string
    QRegExp versionFormat("[0-9]{2}.[0-9]{2}.[0-9]{2}", Qt::CaseInsensitive);
    versionString.contains(versionFormat);

    windowTitle.append(versionFormat.cap(0));
    setWindowTitle(windowTitle);

    resize(500, 500);
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    MainWindow mainWindow;
    mainWindow.showMaximized();
    return app.exec();
}
