/*
 *      A widget containing a slider and two buttons to increase or decrease the value of
 *      the slider. The slider also has a descriptive label showing the current value above
 *      it.
 */

/*
 * Copyright 2011 - 2019 Zytronic Displays Limited, UK.
 *
 * This file is part of the ZyConfig application.
 *
 * ZyConfig is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ZyConfig is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with ZyConfig.
 *
 * If not, see <https://www.gnu.org/licenses/>.
 */


#ifndef BIGLABELEDSLIDER_H
#define BIGLABELEDSLIDER_H

#include <QWidget>
#include <QtWidgets>

#define SLIDER_WIDTH    (420)
#define READING_HGHT    (26)
#define BUTTON_SIZE     (50)
#define SPACER_SIZE     (30)

class BigLabeledSlider : public QWidget
{
    Q_OBJECT
public:
    BigLabeledSlider(QString sliderLabel, int rangeStart, int rangeEnd, int pageStep, QString inc, QString dec,
                     QKeySequence *incKey = 0, QKeySequence *decKey = 0, QLabel *imageLabel = 0, QWidget *parent = 0);

    void setValue(int value);
    void setEnabled(bool value);
    int  getValue();
    void showOverlay();
    void hideOverlay();
    void LabelValTranslator( char*(*func) ( int val) );

    void enableReading(bool);
    void setReading(int);

signals:
    void valueChanged(int value);

private slots:
    void changeLabel(int value);
    void upPressed();
    void downPressed();

private:
    void createWidgets(int rangeStart, int rangeEnd, int pageStep, QString inc, QString dec, QKeySequence *incKey, QKeySequence *decKey);
    void createConnections();
    QLayout * createLayout();

    QLabel       *label,        *mainLabel;
    QPushButton  *up,           *down;
    QSlider      *slider;
    QString      labelBase;
    QShortcut    *shortcut;
    QLabel       *readingLabel;

    QProgressBar *curReading;

    QLabel *decOverlay,         *incOverlay;

    char*(*labelTranslator) (int val);
};

#endif // BIGLABELEDSLIDER_H
