/*
 *   Panel to hold the initial installation activity launching buttons
 */

/*
 * Copyright 2011 - 2019 Zytronic Displays Limited, UK.
 *
 * This file is part of the ZyConfig application.
 *
 * ZyConfig is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ZyConfig is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with ZyConfig.
 *
 * If not, see <https://www.gnu.org/licenses/>.
 */


#ifndef InstallWizard_H
#define InstallWizard_H

#include <QtWidgets>
#include "ZytContentFrame.h"
#include "CalibrationDialog.h"
#include "Z2IntegTests.h"
#include "SetupSensDialog_100.h"
#include "SetupSensDialog_110.h"
#include "SetupSensDialog_MT.h"
#include "Zxy100Data.h"
#include "MonitorSelect.h"

class InstallWizard : public ZytContentFrame
{
    Q_OBJECT
public:
    explicit    InstallWizard       (QWidget *parent);
    void        setAppVersionStrs   (QString &s);
    void        setLEDPoll          (bool *inhibit);
    void        setZXY100Data       (Zxy100Data *dd);

signals:
    void        reReadAxesReq       ();
    void        reReadThresholds    ();
    void        pauseConnectionCheck(bool pause);


public slots:
//  void        updateConnection(const char *t);

private slots:
    void        execIntegTestDialog ();
    void        execSenseDialog     ();
    void        execTestDialog      ();
    void        execCaliDialog      ();

private:
    void        createWidgets       ();
    void        enableButtons       (bool);
    QLayout *   createLayout        ();
    void        createConnections   ();
    char    *   getDevPath          (void);

    bool        getMonName          (char const *key, char *monitorName, int len);
    char    *   getDevPath          (char const *cpuID);



    int16_t     devPID;

    QLabel  *   connectedLabel;

    QLabel      * arrowImage;
    QPixmap       arrowPixMap;
    QLabel      * prompt0,    *prompt1,    *prompt2,     *prompt3;
    QPushButton * step0,      *step1,      *step2,       *step3;
    QLabel      * step0Label, *step1Label, *step2Label,  *step3Label;

    static QTime        buttonPressHoldoff;

    Zxy100Data       *  zxy100Data;

    Z2IntegTests        z2integTests;

    // sensitivity dialog
    SetupSensDialog_BASE   *setupSensDialog;

    CalibrationDialog   caliDialog;
    bool             *  inhibitLEDPolling;

    MonitorSelect    monitorSelector;
};

#endif // InstallWizard_H
