/*
 *      Raw Data Visualisation Widget (Multi-touch)
 */

/*
 * Copyright 2019 Zytronic Displays Limited, UK.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef MONITORSELECT_H
#define MONITORSELECT_H

#include <stdio.h>
#include <QDialog>
#include <QHBoxLayout>
#include <QGroupBox>
#include <QButtonGroup>

// #include "Monitor.h"
#include "DesktopImage.h"

#include <stdint.h>

class MonitorSelect : public QDialog
{
    Q_OBJECT

public:
    MonitorSelect(QWidget *parent = 0);

public slots:
    int exec();

public:
    void setPreferred           (QString name);
    bool selectByName           (QString name);
    void getSelectedMonitor     (Monitor *m);
    void getMonitorListAsRegExp (QRegExp *rExp);

private slots:
    void UpdateMonitorList      (void);
    void updateDesktop          (QAbstractButton *b);

private:
    QString             preferred;
    Monitor             *Screen;
    Monitor             *Primary;
    Monitor             *selected;

    QVector<Monitor*>    monitors;

    QVBoxLayout         *vLayout;
    QHBoxLayout         *hLayout;
    QGroupBox           *monitorBox;
    QButtonGroup        *monitorBGrp;
    QVBoxLayout         *bGrpLayout;

    QHBoxLayout         *buttonLayout;
    QPushButton         *cancelButton;
    QPushButton         *selectButton;
    QLabel              *primaryMsg;

    DesktopImage        *desktopImage;
};

#endif // MONITORSELECT_H
