/*
 *      Threshold Setting Widget. Displays a number of points on the screen in
 *      order to evaluate touch sensitivity data and then write it to the
 *      controller.
 */

/*
 * Copyright 2011 - 2019 Zytronic Displays Limited, UK.
 *
 * This file is part of the ZyConfig application.
 *
 * ZyConfig is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ZyConfig is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with ZyConfig.
 *
 * If not, see <https://www.gnu.org/licenses/>.
 */


#ifndef SETUPSENSDIALOG_MT_H
#define SETUPSENSDIALOG_MT_H

#include "SetupSensDialog_BASE.h"

#include "services.h"
#include "Zxy100Data.h"


// ===  CLASS DEFINITION ============================================

class SetupSensDialog_MT : public SetupSensDialog_BASE
{
Q_OBJECT

    // MT First Touch Mode constants
    const uint16_t ftm_OnThresholdSafetyMargin = 10;       // Add 10
    const float    ftm_OffThresholdSafetyMargin = 1.4f;    // Add 40%


    // enums can be negative! force >= 0, this also allows a simple "toString" service.
    typedef enum CellType_t
    {
        CT_NONE = 0, CT_7,  CT_10, CT_12, CT_15, CT_19,
                     CT_22, CT_32, CT_46, CT_55, CT_65, CT_72, CT_84,
        CT_ERROR
    } CellType;

    struct mtValues_t
    {
        CellType    cellType;
        int         newThreshRetryCount;

        uint16_t    ftm_Mode;
        uint16_t    ftm_OnMetricMaxValue;
        uint16_t    ftm_OnThreshold;
        uint16_t    ftm_OffMetric;
        uint16_t    ftm_OffMetric2;
        uint16_t    ftm_OffThreshold;
    };
    typedef struct mtValues_t MTvalues;


public:
        SetupSensDialog_MT(QWidget *parent = 0);
        void        setZXY100Data(Zxy100Data *dd);

public slots:
        //int exec();                       see super class

protected:
        //void paintEvent(QPaintEvent*);    see super class
        bool initData2(void);
        void applySettings(void);

private slots:
        //void tick();                      see super class

private:

        MTvalues        mtValues;

        void            updateSetProposal   (void);
        int             getPseudoTouchSignal(void);
        //void            changeState(StateChange transition);  pulled to BASE
        void            changeStateWhenRestoreDone(void);      // ZXY110 ONLY!

        int             getSignalLevel(void);
        void            getSignalLocation(uint16_t *tp_x, uint16_t *tp_y);

        int             storeStateData(void);
        void            getNextState(void);
        int             getStateTO(void);
        void            prepNewState(void);

        const char *    getCellTypeName(CellType c);
        CellType        getCellType150(void);
        CellType        getCellType200(void);
        CellType        getCellType300(void);
        CellType        getCellType500(void);
        CellType        getCellType(void);

        int             getThreshEstimate_MT(void);

        int             getZXY150PalmThreshEstimate(int touchUpperThreshold);
        int             getZXY200PalmThreshEstimate(int touchUpperThreshold);
        int             getZXY300PalmThreshEstimate(int touchUpperThreshold);
        int             get_prwlSmallCells500(int wires, int utVal);
        int             get_prwlLargeCells500(int wires, int utVal);
        int             getZXY500PalmThreshEstimate(int touchUpperThreshold);
        int             getPalmThreshEstimate_MT(int touchUpperThreshold);

        uint16_t        getZXY150InterpolatorBiasEstimate(int touchUpperThreshold);
        uint16_t        getInterpolatorBiasEstimate(int touchUpperThreshold);

        void            partialRestoreFactorySettings();
        bool            errorEventsOccurred(void);
        void            haltRawData();
};

#endif // SETUPSENSDIALOG_MT_H
